#include "nLighting.fxh"

float4x4 g_matWorld : NS_WORLD_MATRIX;
float4x4 g_matView  : NS_VIEW_MATRIX;
float4x4 g_matProj	: NS_PROJ_MATRIX;
Texture g_texture : NS_ALBEDO_TEXTURE;
Texture g_texture2nd : NS_NORMAL_TEXTURE;
NSurfaceDesc g_surfaceMaterial : NS_SURFACE_DESC;
NDirLight g_dirLight;
float4 g_ambient = float4(0.4f, 0.4f, 0.4f, 1.0f);

float g_fMaxHeight;
float g_bUseTexture = 1.0f;

float4x4 g_matTexture;

sampler2D g_texSampler = 
sampler_state
{
	Texture = <g_texture>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
};

sampler2D g_texSampler2nd = 
sampler_state
{
	Texture = <g_texture2nd>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
};

sampler2D g_shadowSample = 
sampler_state
{
	Texture = <g_texture>;
    MinFilter = Linear;  
    MagFilter = Linear;
    MipFilter = Linear;
    AddressU  = Clamp;
    AddressV  = Clamp;
};

struct InterShader
{
	float2 texCoord	  : TEXCOORD0;	// Wspolrzedne tekstury
    float3 viewPos    : TEXCOORD1;	// Pozycja w przestrzeni oka
	float3 viewNormal : TEXCOORD2;	// Normalna w przestrzeni oka

	float4 color	  : COLOR0;		// Kolor ambient + emissive
};

float4 VS_func(float4 vPos : POSITION,
				float3 vNormal : NORMAL, 
				float2 vTex : TEXCOORD0,
				out InterShader is) : POSITION
{
	float4x4 mWorldView = mul(g_matWorld, g_matView);
	float4x4 mWorldViewProj = mul(mWorldView, g_matProj);

	is.viewPos = mul(vPos, mWorldView).xyz;							// Transformacja pozycji do przestrzeni widoku
	is.viewNormal = normalize(mul(vNormal, (float3x3)mWorldView));	// Transformacja normalnej do przestrzeni widoku
	is.color = g_surfaceMaterial.ambient * g_ambient;
	is.texCoord = vTex;

	return mul(vPos, mWorldViewProj);
}

float4 PS_func(InterShader is) : COLOR0
{
	float4 diffuse, specular;
	CalculateDirLighting(is.viewPos, is.viewNormal, g_surfaceMaterial.power, g_dirLight, diffuse, specular);
	float4 output = is.color + g_surfaceMaterial.diffuse * diffuse + g_surfaceMaterial.specular * specular;
	output *= !g_bUseTexture ? 1.0f : tex2D(g_texSampler, is.texCoord);
	output.a = 1.0f;
	return output;
}

technique Standard
{
	pass
	{	
		VertexShader = compile vs_2_0 VS_func();
		PixelShader = compile ps_2_0 PS_func();
	}	
}

/****************************************************************/
/****************************************************************/
/****************************************************************/

// Heightmap
float4 VS_func2(float4 vPos : POSITION,
				float3 vNormal : NORMAL, 
				float2 vTex : TEXCOORD0,
				out InterShader is,
				out float pBlend : TEXCOORD3) : POSITION
{
	pBlend = (vPos.y / g_fMaxHeight) - 0.5f;

	float4x4 mWorldView = mul(g_matWorld, g_matView);
	float4x4 mWorldViewProj = mul(mWorldView, g_matProj);

	is.viewPos = mul(vPos, mWorldView).xyz;							// Transformacja pozycji do przestrzeni widoku
	is.viewNormal = normalize(mul(vNormal, (float3x3)mWorldView));	// Transformacja normalnej do przestrzeni widoku
	is.color = g_surfaceMaterial.ambient * g_ambient;
	is.texCoord = vTex;

	return mul(vPos, mWorldViewProj);
}

float4 PS_func2(InterShader is,
				float fBlendFactor : TEXCOORD3) : COLOR0
{
	float4 diffuse, specular;
	CalculateDirLighting(is.viewPos, is.viewNormal, g_surfaceMaterial.power, g_dirLight, diffuse, specular);
	float4 output = is.color + g_surfaceMaterial.diffuse * diffuse + g_surfaceMaterial.specular * specular;
	output *= !g_bUseTexture ? 1.0f : (tex2D(g_texSampler, is.texCoord) * (1.0f - fBlendFactor) + tex2D(g_texSampler2nd, is.texCoord) * fBlendFactor);
	output.a = 1.0f;
	return output;
}

technique Heightmap
{
	pass
	{	
		VertexShader = compile vs_2_0 VS_func2();
		PixelShader = compile ps_2_0 PS_func2();
	}	
}

/****************************************************************/
/****************************************************************/
/****************************************************************/

// Generating Shadowmap
float4 VS_Shadow(float4 vPos : POSITION) : POSITION
{
	float4x4 mWorldView = mul(g_matWorld, g_matView);
	float4x4 mWorldViewProj = mul(mWorldView, g_matProj);

	return mul(vPos, mWorldViewProj);
}

float4 PS_Shadow() : COLOR0
{
	return float4(0.0f, 0.0f, 0.0f, 1.0f);
}

technique ShadowPass
{
	pass
	{	
		VertexShader = compile vs_2_0 VS_Shadow();
		PixelShader = compile ps_2_0 PS_Shadow();

		ZEnable = true;
		ZWriteEnable = true;
		ColorWriteEnable = 0;
	}	
}


/****************************************************************/
/****************************************************************/
/****************************************************************/

// Draw Shadowmap
float4 VS_DrawShadow(float4 vPos : POSITION,
					out float4 pTexCoordProj : TEXCOORD0) : POSITION
{
	float4x4 mWorldView = mul(g_matWorld, g_matView);
	float4x4 mWorldViewProj = mul(mWorldView, g_matProj);
	
	pTexCoordProj = mul(vPos, mul(g_matWorld, g_matTexture));

	return mul(vPos, mWorldViewProj);
}

float4 PS_DrawShadow(float4 pTexCoordProj : TEXCOORD0) : COLOR0
{
	float3 oColor = tex2Dproj(g_shadowSample, pTexCoordProj).rgb;
	return float4(oColor, (oColor.g == 0.0f) * 0.5f);
}

technique DrawShadowPass
{
	pass
	{	
		VertexShader = compile vs_2_0 VS_DrawShadow();
		PixelShader = compile ps_2_0 PS_DrawShadow();

		ZEnable = true;
		ZWriteEnable = false;
		AlphaBlendEnable = true;
		SrcBlend = SrcAlpha;
		DestBlend = InvSrcAlpha;
	}	
}